import math

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import fsmMgr


import edgesBDG
import statePyWeekSplash

gEdgeList={}


#seconds to scan the height of the screen
SCAN_SPEED=1.6

SCAN_INTERVAL=4

BRIGHT_COLOR=(0.7, 1, 0.7)
DIM_COLOR=(0.0, 0.55, 0.0)
FADE_TIME=0.4

SHOW_DELAY=2.0

def genEdgeList():
    for i in range(0,600,SCAN_INTERVAL):
        y=i-300
        x=math.sqrt(300*300-y*y)
        gEdgeList[i]=[400-x,400+x]
    
class StateBDGSplash(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)
        self.angle=0 #east, I guess

        self.clicked=False

        #genEdgeList()
        self.mRasterPos=600
        self.splashTime=0.0
        self.rasterSpeed=600.0/SCAN_SPEED

        global gEdgeList
        gEdgeList=edgesBDG.lines

    def update(self, dt):

        dt=dt/1000.0
        self.splashTime+=dt

        self.mRasterPos-=self.rasterSpeed*dt

        if self.splashTime>SCAN_SPEED+SHOW_DELAY:
            self.done()

    def draw(self):
        for i in gEdgeList.keys():
            if i<self.mRasterPos:
                continue

            distFromRaster=i-self.mRasterPos
            secondsSinceRaster=distFromRaster/self.rasterSpeed

            if secondsSinceRaster>FADE_TIME:
                glColor3f(*DIM_COLOR)
            else:
                br,bg,bb=BRIGHT_COLOR
                dr,dg,db=DIM_COLOR

                fracSinceRaster=secondsSinceRaster/FADE_TIME

                interpR=(dr-br)*fracSinceRaster+br
                interpG=(dg-bg)*fracSinceRaster+bg
                interpB=(db-bb)*fracSinceRaster+bb
                
                glColor3f(interpR, interpG, interpB)
                
                
            glBegin(GL_LINES)


            
            crossingList=gEdgeList[i][:]
            
            while crossingList:
                end1=crossingList.pop()
                end2=crossingList.pop()
                glVertex(end1,i)
                glVertex(end2,i)

            glEnd()


    def handleMouseButton(self, bDown, buttonIndex):
        #calibrate power next

        if bDown==True and buttonIndex==1:
            self.clicked=True
        if bDown==False and buttonIndex==1 and self.clicked:
            self.done()
        return True

    def handleKey(self, key):
        self.done()
        return True

    def done(self):
        fsmMgr.popState()
        fsmMgr.pushState(statePyWeekSplash.StatePyWeekSplash(None))
    

