import math

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import fsmMgr
import textDraw
import stateTitleSplash

import edgesPW4_1 
import edgesPW4_2 


#seconds to scan the height of the screen
SCAN_SPEED=1.6

SCAN_INTERVAL=4

BRIGHT_COLOR=(0.7, 1, 0.7)
DIM_COLOR_1=(0.0, 0.75, 0.0)
DIM_COLOR_2=(0.0, 0.55, 0.0)
FADE_TIME=0.4

SHOW_DELAY=3.5



class StatePyWeekSplash(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)
        self.angle=0 #east, I guess

        self.clicked=False

        #genEdgeList()
        self.mRasterPos1=600
        self.mRasterPos2=600
        self.splashTime=0.0
        self.rasterSpeed=600.0/SCAN_SPEED


    def update(self, dt):
        dt=dt/1000.0
        self.splashTime+=dt

        self.mRasterPos1-=self.rasterSpeed*dt
        if self.splashTime>SCAN_SPEED*0.5:
            self.mRasterPos2-=self.rasterSpeed*dt

        if self.splashTime > SCAN_SPEED+SHOW_DELAY:
            self.done()

    def draw(self):
        self.drawFirstPass()
        self.drawSecondPass()

        c=self.computeColor(0.0, self.splashTime)
        textDraw.drawCenteredString(400,500,c,10,"An entry in")
        textDraw.drawCenteredString(400,175,c,6,"PyWeek 4 game development challenge")
        textDraw.drawCenteredString(400,125,c,6,"http://www.pyweek.org/4/")
        textDraw.drawCenteredString(400, 75,c,6,"entry BDG_07_04")

    def drawFirstPass(self):
        for i in edgesPW4_1.lines.keys():
            if i<self.mRasterPos1:
                continue

            distFromRaster=i-self.mRasterPos1
            secondsSinceRaster=distFromRaster/self.rasterSpeed

            if secondsSinceRaster>FADE_TIME:
                glColor3f(*DIM_COLOR_2)
            else:
                br,bg,bb=BRIGHT_COLOR
                dr,dg,db=DIM_COLOR_2

                fracSinceRaster=secondsSinceRaster/FADE_TIME

                interpR=(dr-br)*fracSinceRaster+br
                interpG=(dg-bg)*fracSinceRaster+bg
                interpB=(db-bb)*fracSinceRaster+bb
                
                glColor3f(interpR, interpG, interpB)
                
                
            glBegin(GL_LINES)
            
            crossingList=edgesPW4_1.lines[i][:]
            
            while crossingList:
                end1=crossingList.pop()
                end2=crossingList.pop()
                glVertex(end1,i)
                glVertex(end2,i)

            glEnd()


    def drawSecondPass(self):
        for i in edgesPW4_2.lines.keys():
            if i<self.mRasterPos2:
                continue

            distFromRaster=i-self.mRasterPos2
            secondsSinceRaster=distFromRaster/self.rasterSpeed

            if secondsSinceRaster>FADE_TIME:
                glColor3f(*DIM_COLOR_1)
            else:
                br,bg,bb=BRIGHT_COLOR
                dr,dg,db=DIM_COLOR_1

                fracSinceRaster=secondsSinceRaster/FADE_TIME

                interpR=(dr-br)*fracSinceRaster+br
                interpG=(dg-bg)*fracSinceRaster+bg
                interpB=(db-bb)*fracSinceRaster+bb
                
                glColor3f(interpR, interpG, interpB)
                
                
            glBegin(GL_LINES)
            
            crossingList=edgesPW4_2.lines[i][:]
            
            while crossingList:
                end1=crossingList.pop()
                end2=crossingList.pop()
                glVertex(end1,i)
                glVertex(end2,i)

            glEnd()



    def handleMouseButton(self, bDown, buttonIndex):
        #calibrate power next

        if bDown==True and buttonIndex==1:
            self.clicked=True
        if bDown==False and buttonIndex==1 and self.clicked:
            self.done()
        return True

    def handleKey(self, key):
        self.done()
        return True

    def done(self):
        fsmMgr.popState()
        fsmMgr.pushState(stateTitleSplash.StateTitleSplash(None))


    def computeColor(self, initTime, timeNow):
        elapsedTime=timeNow-initTime
        
        if elapsedTime>FADE_TIME:
            return DIM_COLOR_2
        else:
            br,bg,bb=BRIGHT_COLOR
            dr,dg,db=DIM_COLOR_2

            fracSinceRaster=elapsedTime/FADE_TIME

            interpR=(dr-br)*fracSinceRaster+br
            interpG=(dg-bg)*fracSinceRaster+bg
            interpB=(db-bb)*fracSinceRaster+bb

            return (interpR, interpG, interpB)
                
                
        
