from OpenGL.GL import *
from OpenGL.GLU import *

font={'A':(4,6,9,10,11,17),
      'B':(0,2,6,9,10,11,16,18),
      'C':(1,4,12,19),
      'D':(0,2,6,11,16,18),
      'E':(0,1,2,9,11,18,19),
      'F':(0,1,2,9,11),
      'G':(1,4,10,12,17,19),
      'H':(2,8,9,10,11,17),
      'I':(0,1,5,14,18,19),
      'J':(8,12,17,19),
      'K':(2,7,9,11,15),
      'L':(2,11,18,19),
      'M':(2,3,7,8,11,17),
      'N':(2,3,8,11,15,17),
      'O':(4,6,12,16),
      'P':(0,2,6,9,10,11),
      'Q':(4,6,12,16,15),
      'R':(0,2,6,9,10,11,15),
      'S':(1,4,9,10,16,18),
      'T':(0,1,5,14),
      'U':(2,8,11,17,18,19),
      'V':(2,7,11,13),
      'W':(2,8,11,13,15,17),
      'X':(3,7,13,15),
      'Y':(3,7,14),
      'Z':(0,1,7,13,18,19),
      '0':(0,1,2,7,8,11,13,17,18,19),
      '1':(4,5,14,18,19),
      '2':(0,6,9,10,11,18,19),
      '3':(0,1,7,10,16,18),
      '4':(4,8,9,10,17),
      '5':(0,1,2,9,10,16,18),
      '6':(1,4,9,10,11,17,18,19),
      '7':(0,1,7,9,10,13),
      '8':(0,1,2,8,9,10,11,17,18,19),
      '9':(0,1,2,8,9,10,16,18),
      '/':(7,13),
      '_':(18,19),
      '-':(9,10),
      '~':(), # fixed-width space
      }

def drawDot(x,y,sz):
    glVertex(x-sz/3,y-sz/3)
    glVertex(x+sz/3,y+sz/3)
    glVertex(x+sz/3,y-sz/3)
    glVertex(x-sz/3,y+sz/3)

    return sz


def drawQuestion(x,y,sz):
    glVertex(sz+x-sz/3,y-sz/3)
    glVertex(sz+x+sz/3,y+sz/3)
    glVertex(sz+x+sz/3,y-sz/3)
    glVertex(sz+x-sz/3,y+sz/3)
    
    glVertex(x,y+2*sz)
    glVertex(x+2*sz,y+2*sz)
    glVertex(x+2*sz,y+2*sz)
    glVertex(x+sz,y+sz)

    return 3*sz

def drawBang(x,y,sz):
    glVertex(x-sz/3,y-sz/3)
    glVertex(x+sz/3,y+sz/3)
    glVertex(x+sz/3,y-sz/3)
    glVertex(x-sz/3,y+sz/3)

    glVertex(x,y+sz)
    glVertex(x,y+2*sz)

    return sz

def drawColon(x,y,sz):
    glVertex(x-sz/3,y-sz/3)
    glVertex(x+sz/3,y+sz/3)
    glVertex(x+sz/3,y-sz/3)
    glVertex(x-sz/3,y+sz/3)

    glVertex(x-sz/3,y-sz/3+sz)
    glVertex(x+sz/3,y+sz/3+sz)
    glVertex(x+sz/3,y-sz/3+sz)
    glVertex(x-sz/3,y+sz/3+sz)

    return sz

def drawSingleQuote(x,y,sz):
    glVertex(x-sz/3,y-sz/3+sz*2)
    glVertex(x+sz/3,y+sz/3+sz*2)
    glVertex(x+sz/3,y-sz/3+sz*2)
    glVertex(x-sz/3,y+sz/3+sz*2)

    return sz



puncFuncs={'.':drawDot,
           '?':drawQuestion,
           '!':drawBang,
           ':':drawColon,
           "'":drawSingleQuote,}




segments=[(0,1),
          (1,2),
          (0,3),
          (0,4),
          (1,3),
          (1,4),
          (1,5),
          (2,4),
          (2,5),
          (3,4),
          (4,5),
          (3,6),
          (3,7),
          (4,6),
          (4,7),
          (4,8),
          (5,7),
          (5,8),
          (6,7),
          (7,8)]


      
      

def drawPoint(x,y,sz,pointIndex):
    stepY,stepX=divmod(pointIndex,3)
    glVertex(x+sz*stepX, y+sz*(2-stepY))


def drawChar(x, y, sz, theChar):
    for segmentIndex in font[theChar]:
        points=segments[segmentIndex]
        drawPoint(x,y,sz,points[0])
        drawPoint(x,y,sz,points[1])


def drawString(x,y,c,sz,theString):
    px=x
    py=y

    glColor(*c)
    glBegin(GL_LINES)
    for theChar in theString.upper():
        if theChar==" ":
            px+= sz*2
        elif theChar in font:
            drawChar(px, py, sz, theChar)
            px += sz*3
        elif theChar in puncFuncs:
            px+=puncFuncs[theChar](px,py,sz)
    glEnd()

def drawCenteredString(x,y,c,sz,theString):
    w=stringWidth(sz,theString)
    drawString(x-w/2,y+sz,c,sz,theString)
                  


def charWidth(sz, theChar):
    if (theChar in font or
        theChar in '?'):
        return sz*3
    elif theChar==' ':
        return sz*2
    elif theChar in puncFuncs:
        return sz

    return 0
    
def stringWidth(sz, theString):
    return sum(map(lambda theChar:charWidth(sz, theChar),theString.upper()))
